"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
const bodyParser = require("body-parser");
const cookieParser = require("cookie-parser");
const express = require("express");
const serveStatic = require("serve-static");
const bearerToken = require("express-bearer-token");
const http = require("http");
const access_token_1 = require("../models/access-token");
const routes_1 = require("../routes");
const utils_1 = require("./utils");
const path = require("path");
const uuidv4 = require('uuid/v4'); // tslint:disable-line
let manifest = null;
try {
    manifest = require('../../public/dist/manifest.json');
}
catch (e) {
    // noop
}
const devMode = process.env.APP_ENV === 'development';
const httpLogger = require('pino-http')({
    logger: utils_1.logger,
    genReqId: uuidv4,
    serializers: {
        req: (req) => {
            if (devMode) {
                return `${req.method} ${req.url} [${req.id}]`;
            }
            else {
                return {
                    id: req.id,
                    method: req.method,
                    url: req.url,
                    headers: req.headers,
                    remoteAddress: req.connection && req.connection.remoteAddress,
                    remotePort: req.connection && req.connection.remotePort
                };
            }
        },
        res: (res) => {
            if (devMode) {
                const cleanedHeaders = String(res.header)
                    .replace(/(\r\n)+/g, ' • ')
                    .replace(/\s•\s$/, '');
                return `[${res.statusCode}] ${cleanedHeaders}`;
            }
            else {
                return {
                    statusCode: res.statusCode,
                    header: res.header
                };
            }
        }
    }
});
function generateServer() {
    const app = express();
    const router = express.Router(); // eslint-disable-line new-cap
    app.disable('x-powered-by');
    app.use(httpLogger);
    app.use(utils_1.sentry.requestHandler());
    app.use(bodyParser.json());
    app.use(bearerToken());
    app.use((req, res, next) => {
        if (req.token) {
            return access_token_1.default
                .query()
                .where({ token: req.token })
                .first()
                .then((token) => {
                if (token) {
                    req.user = {
                        id: token.id,
                        type: 'robot',
                        name: token.name,
                        roles: token.roles
                    };
                    return next();
                }
                else {
                    return res.status(401).send({ error: 'Incorrect auth token' });
                }
            })
                .catch(next);
        }
        else {
            req.user = {
                id: null,
                roles: {}
            };
            return next();
        }
    });
    app.use(cookieParser());
    if (devMode) {
        app.use(serveStatic(path.resolve(__dirname, '..', '..', 'public')));
    }
    routes_1.default(app);
    app.use('/', router);
    app.use(utils_1.sentry.errorHandler());
    app.use((err, req, res, next) => {
        if (req.log) {
            req.log.error(err, 'Unhandled error');
        }
        else {
            utils_1.reportError(err, { message: 'Unhandled error' });
        }
        res.status(500).send('Internal server error');
    });
    return function listen(port) {
        const server = http.createServer(app);
        server.on('listening', () => {
            utils_1.logger.info(`Express server is listening on port ${server.address().port}`);
        });
        server.on('error', error => {
            if (error.syscall !== 'listen') {
                throw error;
            }
            switch (error.code) {
                case 'EACCES':
                    utils_1.logger.error(`Port ${port} requires elevated privileges`);
                    process.exit(1);
                    break;
                case 'EADDRINUSE':
                    utils_1.logger.error(`Port ${port} is already in use`);
                    process.exit(1);
                    break;
                default:
                    throw error;
            }
        });
        server.listen(port);
    };
}
exports.default = generateServer;
//# sourceMappingURL=express.js.map